<?php
/*
Plugin Name: WordPress File Monitor
Plugin URI: http://mattwalters.net/projects/wordpress-file-monitor/
Description: Monitor files under your WordPress installation for changes
Author: Matt Walters
Version: 1.1
Author URI: http://mattwalters.net/
*/ 

if (!class_exists('msw_WordPressMonitor')) {
	class msw_WordPressMonitor {
		// Define possible options, PHP4 compatible
		var $options = array(
			'scan_interval',
			'from_address',
			'notify_address',
			'site_root',
			'exclude_paths',
			'modification_detection',
			'notification_format'
		);

		function msw_WordPressMonitor() { $this->__construct(); } // PHP4 compatibility
		
		function __construct() {
			add_action("admin_menu", array(&$this,"add_admin_pages")); // Call to add menu option in admin

			// Assumes your language files will be in the format: wordpress_file_monitor-locationcode.mo
			$wordpress_file_monitor_locale = get_locale();
			$wordpress_file_monitor_mofile = dirname(__FILE__) . "/languages/wordpress_file_monitor-".$wordpress_file_monitor_locale.".mo";
			load_textdomain("wordpress_file_monitor", $wordpress_file_monitor_mofile);
		}

		function add_admin_pages() { // Add menu option in admin
			add_submenu_page('options-general.php', "WordPress File Monitor", "WordPress File Monitor", 10, "WordPress File Monitor", array(&$this,"output_sub_admin_page_0"));
		}

		function update_options($newOptions) { 
			foreach ($this->options as $option) { // Loop through post variables and get form fields corresponding to valid settings
				$options[$option] = $newOptions[$option];
			}
			if (!get_option('wpfm_options')) { add_option('wpfm_options'); } // Add option if it does not exist
			update_option('wpfm_options', maybe_serialize($options)); // Set settings to new values
		}

		function output_sub_admin_page_0() { // Form to configure plugin
			$options = maybe_unserialize(get_option('wpfm_options'));
			?>
			<div class="wrap">
				<h2>WordPress File Monitor Options</h2>
				<br/><br/>
				<form name="msw_wpfm_options" action="<?php echo get_bloginfo('wpurl'); ?>/wp-admin/options-general.php?page=WordPress%20File%20Monitor" method="post">
					<?php if (function_exists('wp_nonce_field')) { wp_nonce_field('wpfm-update-options'); } ?>
					<input type="hidden" name="msw_wpfm_action" value="1" id="msw_wpfm_action">
					<table>
						<tr>
							<td valign="middle"><label for="scan_interval"><?php _e('Scan Interval', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="scan_interval" value="<?php echo @$options['scan_interval']; ?>" id="scan_interval"> (<?php _e('in minutes', 'wordpress_file_monitor'); ?>)</td>
						</tr>
						<tr>
							<td valign="middle"><label for="modification_detection"><?php _e('Detection Method', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle">
								<select name="modification_detection" id="modification_detection">
									<option value="datetime"<?php if (@$options['modification_detection'] == 'datetime') { echo ' selected'; } ?>><?php _e('Modification Date (faster, but less secure)', 'wordpress_file_monitor'); ?></option>
									<option value="md5"<?php if (@$options['modification_detection'] == 'md5') { echo ' selected'; } ?>><?php _e('Hash (more secure, but takes longer)', 'wordpress_file_monitor'); ?></option>
								</select>
							</td>
						</tr>
						<tr>
							<td valign="middle"><label for="from_address"><?php _e('From Address', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="from_address" value="<?php echo @$options['from_address']; ?>" id="from_address"> (<?php _e('for alerts', 'wordpress_file_monitor'); ?>)</td>
						</tr>
						<tr>
							<td valign="middle"><label for="notify_address"><?php _e('Notify Address', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="notify_address" value="<?php echo @$options['notify_address']; ?>" id="notify_address"> (<?php _e('for alerts', 'wordpress_file_monitor'); ?>)</td>
						</tr>
						<tr>
							<td valign="middle"><label for="notification_format"><?php _e('Notification Format', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle">
								<select name="notification_format" id="notification_format">
									<option value="detailed"<?php if (@$options['notification_format'] == 'detailed') { echo ' selected'; } ?>><?php _e('Detailed', 'wordpress_file_monitor'); ?></option>
									<option value="subversion"<?php if (@$options['notification_format'] == 'subversion') { echo ' selected'; } ?>><?php _e('Subversion (A/M/D)', 'wordpress_file_monitor'); ?></option>
								</select>
							</td>
						</tr>
						<tr>
							<td valign="middle"><label for="site_root"><?php _e('Site Root', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="middle"><input type="text" name="site_root" value="<?php if ($options['site_root'] == '') { echo ABSPATH; } else { echo $options['site_root']; } ?>" id="site_root"> (<?php _e('Default', 'wordpress_file_monitor'); ?>: <?php echo ABSPATH; ?>)</td>
						</tr>
						<tr>
							<td valign="top"><label for="exclude_paths"><?php _e('Exclude Paths', 'wordpress_file_monitor'); ?>: </label></td>
							<td valign="top"><textarea name="exclude_paths" rows="8" cols="40"><?php echo @$options['exclude_paths']; ?></textarea></td>
						</tr>
						<tr>
							<td valign="top">&nbsp;</td>
							<td valign="top">
								<?php _e('Do not include the site root from above. One path per line.', 'wordpress_file_monitor'); ?><br/>
								<br/>
								<?php _e('If you run any kind of cacheing plugins or scripts on your site and the cache files are stored in a folder under the Site Root specified above, ', 'wordpress_file_monitor'); ?><br/>
								<?php _e('it is HIGHLY recommended you exclude the paths to your cache directories.', 'wordpress_file_monitor'); ?>
								<br/><br/>
								<?php _e('Examples', 'wordpress_file_monitor'); ?>:<br/>
								wordpress/wp-content/cache<br/>
								wp-content/uploads
							</td>
						</tr>
					</table>
					<input type="submit" name="Submit" value="<?php _e('Submit', 'wordpress_file_monitor'); ?>" id="Submit">
				</form>
			</div>
			<?php
		}

		function directoryToArray($directory, $recursive=true) { // Obtain recursive file/directory listing
			$array_items = array();
			if ($handle = opendir($directory)) {
				while (false !== ($file = readdir($handle))) {
					if ($file != "." && $file != "..") {
						if (is_dir($directory. "/" . $file)) {
							if($recursive) {
								$array_items = array_merge($array_items, $this->directoryToArray($directory. "/" . $file, $recursive));
							}
							$file = $directory . "/" . $file;
							$array_items[] = preg_replace("/\/\//si", "/", $file);
						} else {
							$file = $directory . "/" . $file;
							$array_items[] = preg_replace("/\/\//si", "/", $file);
						}
					}
				}
				closedir($handle);
			}
			return $array_items;
		}

		function scan_site() { // Perform scan
			$options = maybe_unserialize(get_option('wpfm_options')); // Get settings
			$dirListing = $this->directoryToArray($options['site_root']); // Get recursive file/directory listing
			foreach ($dirListing as $item) { // Loop through listing and remove files within exclude paths
				$excludePaths = explode("\n", $options['exclude_paths']);
				$scanPath = true;
				foreach ($excludePaths as $excludePath) {
					$temp = $options['site_root'] . trim($excludePath);
					if (strpos($item, $temp) !== false) { $scanPath = false; } // File is in exclude path, ignore it
				}
				if ($scanPath) {
					/*
						Set up an array of files.  The array is:
						FILENAME => HASH [OR] TIMESTAMP
						
						The user has the ability to configure which scanning method they would like to use.  Based on their choice
						either an md5 hash or the files timestamp will be set as the value to later be tested against.  If they change
						methods, the next time a scan is run, it will appear as though every file has been changed due to this setup.
						
						... that's life :)
					*/
					if ($options['modification_detection'] == 'md5') { // Test for changes to file via md5 hash
						$currentDirListing[$item] = md5_file($item);
					} else { // Test for changes to file via file timestamp
						$currentDirListing[$item] = filemtime($item);
					}
				}
			}
			$previousDirListing = get_option('wpfm_listing'); // Get serialized array of the previous scan if it exists
			if ($previousDirListing) { // If it did exist ... continue
				$previousDirListing = maybe_unserialize($previousDirListing);
				// Check for differences
				$diff['addedFiles'] = array_diff_key($currentDirListing, $previousDirListing); // If files were added, create array of those files
				$diff['removedFiles'] = array_diff_key($previousDirListing, $currentDirListing); // If files were removed, create array of those files
				$diff['changedFiles'] = array_diff($currentDirListing, $previousDirListing); // Compare previous scan to this scan, create array of files changed
				foreach ($diff['addedFiles'] as $file=>$v) { // Remove list of added files from changed files to prevent duplication in the email
					unset($diff['changedFiles'][$file]);
				}
				foreach ($diff['removedFiles'] as $file=>$v) { // Remove list of deleted files from changes files to prevent duplication in the email
					unset($diff['changedFiles'][$file]);
				}
				update_option('wpfm_listing', maybe_serialize($currentDirListing)); // Save this scan to be compared against in next scan
				if (count($diff['addedFiles']) > 0 || count($diff['removedFiles']) > 0 || count($diff['changedFiles']) > 0) {
					$this->notify($diff); // Trigger notification email
				}
			} else {
				// This is the first scan, so add the option and set its value to be a serialized array of the recursive listing
				add_option('wpfm_listing');
				update_option('wpfm_listing', maybe_serialize($currentDirListing));
			}
		}

		function notify($diff=array()) {
			// Send notifaction email
			$options = maybe_unserialize(get_option('wpfm_options')); // Get settings
			$toEmail = $options['notify_address'];
			$fromEmail = $options['from_address'];
			$fromName = __('WordPress File Monitor', 'wordpress_file_monitor');
			$headers = "From: " . $fromName . " <" . $fromEmail . ">\r\n";
			$subject = __('WordPress File Monitor: Alert', 'wordpress_file_monitor');
			$body = __('This email is to alert you of the following changes to the file system of your website:', 'wordpress_file_monitor');
			$body .= "\n\n";
			
			switch ($options['notification_format']) {
				// Format email according to users settings
				case 'detailed':
					if (count($diff['addedFiles']) > 0) {
						$body .= __('Added:', 'wordpress_file_monitor');
						$body .= "\n";
						foreach ($diff['addedFiles'] as $file=>$timeStamp) {
							$body .= "./" . str_replace($options['site_root'], '', $file) . "\n";
						}
						$body .= "\n\n";
					}
					if (count($diff['removedFiles']) > 0) {
						$body .= __('Removed:', 'wordpress_file_monitor');
						$body .= "\n";
						foreach ($diff['removedFiles'] as $file=>$timeStamp) {
							$body .= "./" . str_replace($options['site_root'], '', $file) . "\n";
						}
						$body .= "\n\n";
					}
					if (count($diff['changedFiles']) > 0) {
						$body .= __('Changed:', 'wordpress_file_monitor');
						$body .= "\n";
						foreach ($diff['changedFiles'] as $file=>$timeStamp) {
							$body .= "./" . str_replace($options['site_root'], '', $file) . "\n";
						}
					}
					break;
				case 'subversion': 
					if (count($diff['addedFiles']) > 0) {
						foreach ($diff['addedFiles'] as $file=>$timeStamp) {
							$body .= "A ./" . str_replace($options['site_root'], '', $file) . "\n";
						}
					}
					if (count($diff['removedFiles']) > 0) {
						foreach ($diff['removedFiles'] as $file=>$timeStamp) {
							$body .= "D ./" . str_replace($options['site_root'], '', $file) . "\n";
						}
					}
					if (count($diff['changedFiles']) > 0) {
						foreach ($diff['changedFiles'] as $file=>$timeStamp) {
							$body .= "M ./" . str_replace($options['site_root'], '', $file) . "\n";
						}
					}
					break;
				default:
					// Really ... no way we should end up here, but just in case ...
					$body = __('There is an error with your configuration of WordPress File Monitor.  You need to specify a notification format.', 'wordpress_file_monitor');
					break;
			}
			mail($toEmail, $subject, $body, $headers); // Send email
		}
	}
}

function msw_wpfm_scan() {
	global $msw_wpfm;
	$options = maybe_unserialize(get_option('wpfm_options')); // Get settings
	$previousScan = get_option('wpfm_previousScan'); // Get previous scan timestamp
	$scanNeeded = false;
	$scanInterval = intval($options['scan_interval']); // Get setting for how often scan should be performed

	if ($previousScan) { // Determine if scan interval has been exceeded
		if (((time() - $previousScan) / 60) > $scanInterval) {
			$scanNeeded = true;
		}
	} else { // Scan has never been run so create option and perform initial scan
		$scanNeeded = true;
		add_option('wpfm_previousScan');
	}
	if ($scanNeeded) { // If scan is needed, perform scan and update last scan timestamp
		$msw_wpfm->scan_site();
		update_option('wpfm_previousScan', time());
	}
}

function msw_wpfm_admin_processing() {
	// Process form in administration area if needed
	if (isset($_POST['msw_wpfm_action'])) {
		global $msw_wpfm;
		check_admin_referer('wpfm-update-options'); // Security check
		$msw_wpfm->update_options($_POST); // Update options based on form submission
	}
}

function msw_plugin_action_links($links, $file) {
	// Add 'Settings' link to plugin listing page in admin
	$plugin_file = 'wordpress-file-monitor/'.basename(__FILE__);
	if ($file == $plugin_file) {
		$settings_link = '<a href="'.get_bloginfo('wpurl').'/wp-admin/options-general.php?page=WordPress%20File%20Monitor">'.__('Settings', 'wordpress_file_monitor').'</a>';
		array_unshift($links, $settings_link);
	}
	return $links;
}

function msw_wpfm_install() {
	// Default settings
	$options = array(
		'scan_interval' => 15,
		'from_address' => get_option('admin_email'),
		'notify_address' => get_option('admin_email'),
		'site_root' => ABSPATH,
		'exclude_paths' => '',
		'modification_detection' => 'datetime',
		'notification_format' => 'detailed'
	);
	if (!get_option('wpfm_options')) { add_option('wpfm_options'); } // Add option if it doesn't exist
	update_option('wpfm_options', maybe_serialize($options)); // Set to default settings
}

if (is_file(trailingslashit(ABSPATH.PLUGINDIR).'wordpress_file_monitor.php')) {
	define('MSW_WPFM_FILE', trailingslashit(ABSPATH.PLUGINDIR).'wordpress_file_monitor.php');
} else if (is_file(trailingslashit(ABSPATH.PLUGINDIR).'wordpress-file-monitor/wordpress_file_monitor.php')) {
	define('MSW_WPFM_FILE', trailingslashit(ABSPATH.PLUGINDIR).'wordpress-file-monitor/wordpress_file_monitor.php');
}

if (!$msw_wpfm) { $msw_wpfm = new msw_WordPressMonitor(); } // Instantiate if needed

add_action('init', 'msw_wpfm_admin_processing'); // Process form submission if needed
add_action('init', 'msw_wpfm_scan'); // Check to see if scan is needed
add_filter('plugin_action_links', 'msw_plugin_action_links', 10, 2); // Add settings link to plugin listing
register_activation_hook(MSW_WPFM_FILE, 'msw_wpfm_install'); // Run install routine if being activated
?>